import requests
import shutil
import time
import argparse
import json
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.chrome.options import Options
from pathlib import Path
import lxml.html
import os
import sys
from fake_useragent import UserAgent
if sys.version_info[0] > 2:
    import urllib.parse as urlparse
else:
    import urlparse
    import io
    reload(sys)
    sys.setdefaultencoding('utf8')

'''
Commandline based Google Images scraping/downloading. Gets up to 1000 images.
Author: Rushil Srivastava (rushu0922@gmail.com)
'''


def search(url):
    # Create a browser and resize depending on user preference

    chrome_options = Options()
    chrome_options.add_argument("--headless")

    browser = webdriver.Chrome(chrome_options=chrome_options)
    browser.set_window_size(1024, 768)
    print("\n===============================================\n")
    print("[%] Successfully launched ChromeDriver")

    # Open the link
    browser.get(url)
    time.sleep(1)
    print("[%] Successfully opened link.")

    element = browser.find_element_by_tag_name("body")

    print("[%] Scrolling down.")
    # Scroll down
    for i in range(30):
        element.send_keys(Keys.PAGE_DOWN)
        time.sleep(0.3)  # bot id protection

    try:
        browser.find_element_by_id("smb").click()
        print("[%] Successfully clicked 'Show More Button'.")
        for i in range(50):
            element.send_keys(Keys.PAGE_DOWN)
            time.sleep(0.3)  # bot id protection
    except:
        for i in range(10):
            element.send_keys(Keys.PAGE_DOWN)
            time.sleep(0.3)  # bot id protection

    print("[%] Reached end of Page.")

    time.sleep(1)
    # Get page source and close the browser
    source = browser.page_source
    if sys.version_info[0] > 2:
        with open('{}/dataset/logs/google/source.html'.format(os.getcwd()), 'w+', encoding='utf-8', errors='replace') as f:
            f.write(source)
    else:
        with io.open('{}/dataset/logs/google/source.html'.format(os.getcwd()), 'w+', encoding='utf-8') as f:
            f.write(source)

    browser.close()
    print("[%] Closed ChromeDriver.")

    return source


def error(link, query):
    print("[!] Skipping {}. Can't download or no metadata.\n".format(link))
    file = Path("{}/dataset/logs/google/errors.log".format(os.getcwd(), query))
    if file.is_file():
        with open("{}/dataset/logs/google/errors.log".format(os.getcwd(), query), "a") as myfile:
            myfile.write(link + "\n")
    else:
        with open("{}/dataset/logs/google/errors.log".format(os.getcwd(), query), "w+") as myfile:
            myfile.write(link + "\n")


def save_image(link, file_path):
    # Use a random user agent header for bot id
    ua = UserAgent(verify_ssl=False)
    headers = {"User-Agent": ua.random}
    r = requests.get(link, stream=True, headers=headers)
    if r.status_code == 200:
        with open(file_path, 'wb') as f:
            r.raw.decode_content = True
            shutil.copyfileobj(r.raw, f)
    else:
        raise Exception("Image returned a {} error.".format(r.status_code))


def download_image(link, image_data, metadata, query):
    download_image.delta += 1

    # Get the image link
    try:
        # Get the file name and type
        file_name = link.split("/")[-1]
        type = file_name.split(".")[-1]
        type = (type[:3]) if len(type) > 3 else type
        if type.lower() == "jpe":
            type = "jpeg"
        if type.lower() not in ["jpeg", "jfif", "exif", "tiff", "gif", "bmp", "png", "webp", "jpg"]:
            type = "jpg"

        # Download the image
        print("[%] Downloading Image #{} from {}".format(
            download_image.delta, link))
        try:
            save_image(link, "{}/dataset/google/{}/".format(os.getcwd(), query) +
                       "Scrapper_{}.{}".format(str(download_image.delta), type))
            print("[%] Downloaded File")
            if metadata:
                with open("{}/dataset/google/{}/Scrapper_{}.json".format(os.getcwd(), query, str(download_image.delta)), "w") as outfile:
                    json.dump(image_data, outfile, indent=4)
        except Exception as e:
            download_image.delta -= 1
            print("[!] Issue Downloading: {}\n[!] Error: {}".format(link, e))
            error(link, query)
    except Exception as e:
        download_image.delta -= 1
        print("[!] Issue getting: {}\n[!] Error:: {}".format(link, e))
        error(link, query)


def google(url, metadata, query, limit):

    source = search(url)

    # set stack limit
    sys.setrecursionlimit(1000000)

    # Parse the page source and download pics
    soup = BeautifulSoup(str(source), "html.parser")

    try:
        os.remove("{}/dataset/logs/google/errors.log".format(os.getcwd()))
    except OSError:
        pass

    # Get the links and image data
    links = [json.loads(i.text)["ou"]
             for i in soup.find_all("div", class_="rg_meta")]
    print("[%] Indexed {} Possible Images.".format(len(links)))
    print("\n===============================================\n")
    print("[%] Getting Image Information.")
    images = {}
    link_counter = 0
    download_image.delta = 0
    for a in soup.find_all("div", class_="rg_meta"):
        # assuming a 25% error rate
        if download_image.delta >= int(limit):
            break
        print("\n------------------------------------------")
        rg_meta = json.loads(a.text)
        if 'st' in rg_meta:
            title = rg_meta['st']
        else:
            title = ""
        link = rg_meta["ou"]
        print("\n[%] Getting info on: {}".format(link))
        try:
            image_data = "google", query, rg_meta["pt"], rg_meta["s"], title, link, rg_meta["ru"]
            images[link] = image_data
            try:
                download_image(link, images[link], metadata, query)
            except Exception as e:
                error(link, query)
        except Exception as e:
            images[link] = image_data
            print("[!] Issue getting data: {}\n[!] Error: {}".format(rg_meta, e))

        link_counter += 1

    print("\n\n[%] Done. Downloaded {} images.".format(download_image.delta))
    print("\n===============================================\n")



from pip import __main__
import sys
import os
import argparse
import _bing
import _google

'''
Console program to download images from Google or Bing.
'''

# By Rushil Srivastava

def main():

    # Parser
    parser = argparse.ArgumentParser(
        description="Scrape images from the internet.")
    parser.add_argument(
        "engine", help="Which search engine should be used? (Bing/Google)")
    parser.add_argument(
        "query", help="Query that should be used to scrape images.")
    parser.add_argument(
        "--limit", help="Amount of images to be scraped.", default=1000, required=False)
    parser.add_argument("--json", help="Should image metadata be downloaded?",
                        action='store_true', required=False)
    parser.add_argument(
        "--url", help="Google: Scrape images from a google image search link", required=False)  # Google Specific
    parser.add_argument("--adult-filter-off", help="Disable adult filter",
                        action='store_true', required=False)  # Bing Specific
    args = parser.parse_args()

    # Variables
    engine = args.engine.lower() if args.engine.lower() is not None else "google"
    query = urlparse.parse_qs(urlparse.urlparse(args.url).query)[
        'q'][0] if args.url is not None else args.query
    limit = args.limit
    metadata = args.json if args.json is not None else False
    adult = "off" if args.adult_filter_off else "on"
    if engine == "google" or engine == "g":
        engine = "google"
        url = args.url if args.url is not None else "https://www.google.com/search?q={}&source=lnms&tbm=isch".format(
            query)
    elif engine == "bing" or engine == "b":
        engine = "bing"
        url = "https://www.bing.com/images/async?q={}&first=0&adlt={}".format(
        str(query), adult)
    else:
    	sys.exit("Invalid engine specified.")

    cwd = os.getcwd()

    # check directory and create if necessary
    if not os.path.isdir("{}/dataset/".format(cwd)):
        os.makedirs("{}/dataset/".format(cwd))
    if not os.path.isdir("{}/dataset/{}/{}".format(cwd, engine, query)):
        os.makedirs("{}/dataset/{}/{}".format(cwd, engine, query))
    if not os.path.isdir("{}/dataset/logs/{}/".format(cwd, engine, query)):
        os.makedirs("{}/dataset/logs/{}/".format(cwd, engine, query))

    if engine == "google":
        _google.google(url, metadata, query, limit)
    else:
        _bing.bing(url, metadata, query, limit, adult)

if __name__ == "__main__":
    main()


import requests
import time
import shutil
import argparse
import json
from bs4 import BeautifulSoup
from pathlib import Path
import lxml.html
import os
import sys
from fake_useragent import UserAgent
if sys.version_info[0] > 2:
    import urllib.parse as urlparse
else:
    import urlparse
    import io
    reload(sys)
    sys.setdefaultencoding('utf8')

'''
Commandline based Bing Images scraping/downloading. Gets unlimited amounts of images.
Author: Rushil Srivastava (rushu0922@gmail.com)
'''


def error(link, query):
    print("[!] Skipping {}. Can't download or no metadata.\n".format(link))
    file = Path("{}/dataset/logs/bing/errors.log".format(os.getcwd(), query))
    if file.is_file():
        with open("{}/dataset/logs/bing/errors.log".format(os.getcwd(), query), "a") as myfile:
            myfile.write(link + "\n")
    else:
        with open("{}/dataset/logs/bing/errors.log".format(os.getcwd(), query), "w+") as myfile:
            myfile.write(link + "\n")


def save_image(link, file_path):
    # Use a random user agent header for bot id
    ua = UserAgent(verify_ssl=False)
    headers = {"User-Agent": ua.random}
    r = requests.get(link, stream=True, headers=headers)
    if r.status_code == 200:
        with open(file_path, 'wb') as f:
            r.raw.decode_content = True
            shutil.copyfileobj(r.raw, f)
    else:
        raise Exception("Image returned a {} error.".format(r.status_code))


def download_image(link, image_data, metadata, query):
    download_image.delta += 1

    # Get the image link
    try:
        # Get the file name and type
        file_name = link.split("/")[-1]
        type = file_name.split(".")[-1]
        type = (type[:3]) if len(type) > 3 else type
        if type.lower() == "jpe":
            type = "jpeg"
        if type.lower() not in ["jpeg", "jfif", "exif", "tiff", "gif", "bmp", "png", "webp", "jpg"]:
            type = "jpg"

        # Download the image
        print("[%] Downloading Image #{} from {}".format(
            download_image.delta, link))
        try:
            save_image(link, "{}/dataset/bing/{}/".format(os.getcwd(), query) +
                       "Scrapper_{}.{}".format(str(download_image.delta), type))
            print("[%] Downloaded File")
            if metadata:
                with open("{}/dataset/bing/{}/Scrapper_{}.json".format(os.getcwd(), query, str(download_image.delta)), "w") as outfile:
                    json.dump(image_data, outfile, indent=4)
        except Exception as e:
            download_image.delta -= 1
            print("[!] Issue Downloading: {}\n[!] Error: {}".format(link, e))
            error(link, query)
    except Exception as e:
        download_image.delta -= 1
        print("[!] Issue getting: {}\n[!] Error:: {}".format(link, e))
        error(link, query)


def bing(url, metadata, query, delta, adult):

    delta = int(delta)

    # set stack limit
    sys.setrecursionlimit(1000000)

    page_counter = 0
    link_counter = 0
    download_image.delta = 0
    while download_image.delta < delta:
        # Parse the page source and download pics
        ua = UserAgent(verify_ssl=False)
        headers = {"User-Agent": ua.random}
        payload = (("q", str(query)), ("first", page_counter), ("adlt", adult))
        source = requests.get(
            "https://www.bing.com/images/async", params=payload, headers=headers).content
        soup = BeautifulSoup(str(source).replace('\r\n', ""), "html.parser")

        try:
            os.remove("dataset/logs/bing/errors.log")
        except OSError:
            pass

        # Get the links and image data
        links = [json.loads(i.get("m").replace('\r\n', ""))["murl"]
                 for i in soup.find_all("a", class_="iusc")]
        print("[%] Indexed {} Images on Page {}.".format(
            len(links), page_counter + 1))
        print("\n===============================================\n")
        print("[%] Getting Image Information.")
        images = {}
        for a in soup.find_all("a", class_="iusc"):
            if download_image.delta >= delta:
                break
            print("\n------------------------------------------")
            iusc = json.loads(a.get("m"))
            link = iusc["murl"]
            print("\n[%] Getting info on: {}".format(link))
            try:
                image_data = "bing", query, link, iusc["purl"], iusc["md5"]
                images[link] = image_data
                try:
                    download_image(link, images[link], metadata, query)
                except Exception as e:
                    error(link, query)
            except Exception as e:
                images[link] = image_data
                print("[!] Issue getting data: {}\n[!] Error: {}".format(rg_meta, e))

            link_counter += 1

        page_counter += 1

    print("\n\n[%] Done. Downloaded {} images.".format(download_image.delta))
    print("\n===============================================\n")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="image_search",
    version="0.0.1",
    author="Rushil Srivastava",
    author_email="rushu0922@gmail.com",
    description="Python Library to download images and metadata from popular search engines.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rushilsrivastava/image_search",
    download_url="https://github.com/rushilsrivastava/image_search/archive/0.0.1.tar.gz",
    keywords = ['google', 'bing', 'images', 'scraping'],
    packages=['image_search'],
    install_requires=[
        'selenium',
        'requests[security]',
        'argparse',
        'pathlib',
        'lxml',
        'bs4',
        'fake_useragent',
    ],
    classifiers=(
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    entry_points={
        'console_scripts':
        ['image_search=image_search.console:main'],
    }
)